#include "Trigger.h"
#include "electric_funkatron.h"
#include "generalmidi.h"
#include "WProgram.h"

#include "midi.h"

#define NOTE_OFF_STATE 0
#define WAIT_ON_VEL_STATE 1
#define NOTE_ON_STATE 2



extern int drumchan;

/*
 * Use this array to manually tune the crosstalk threshold for each trigger.
 * In theory, once it's set it should be fine for all time unless you swap out piezos.
 * It is recommended you set the value as DEFAULT_CROSSTALK_THRESHOLD +/- some value.  This will give
 * a relative idea of how much it has been tuned up or down.
 */
int crosstalk_threshold[16] =
{
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 0 ==> not used
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 1 ==> trigger L
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 2 ==> trigger M
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 3 ==> trigger A
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 4 ==> trigger B
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 5 ==> trigger C
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 6 ==> trigger D
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 7 ==> trigger E
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 8 ==> trigger F
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 9 ==> trigger G
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 10 ==> trigger H
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 11 ==> trigger I
  DEFAULT_CROSSTALK_THRESHOLD + 20,       // pin 12 ==> trigger J
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 13 ==> not used
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 14 ==> not used
  DEFAULT_CROSSTALK_THRESHOLD,       // pin 15 ==> not used
};


void Trigger::init(int p, NOTE n)
{
  this->pin = p;
  this->note = n.note;
  this->state = NOTE_OFF_STATE;
  this->offTime = 0;
  this->threshold = crosstalk_threshold[p];
  this->holdtime = n.hold_time;
  this->vel_scale = n.vel_scale;
}

Trigger::Trigger()
{
  NOTE n = { gm_acoustic_snare, 20, 50 }; 
  init(triggerA, n);
}

Trigger::Trigger(int p, NOTE n)
{
  init(p, n);
}


Trigger::~Trigger()
{
}

// entry point for monitoring a Trigger
void Trigger::process()
{
  //DEBUG("enter: process()");
  checkInput();
  checkNoteOff();
  //DEBUG("exit: process()");
}

// checks the Trigger to see if it has been hit
void Trigger::checkInput()
{
  //DEBUG("enter: checkInput()");
  if(this->state != NOTE_OFF_STATE) return; // don't check if we're currently playing (or waiting to calc the velocity)
  int val = analogRead(pin);
  
#if USE_VELOCITY
  if(val >= threshold)
  {
    int velocity = 0;
    this->state = NOTE_ON_STATE;
    
    val = (val / 4) + this->vel_scale;
    velocity = val > 127 ? 127 : val;
    
    
    //while(val > PIEZOTHRESHOLD)
    //{
    // val = analogRead(pin);
    //  if(velocity < 127) velocity++; // avoid over-flow into what will be interpreted as a signed byte by midi
    //}
    //noteOff(drumchan,this->note); // make sure the channel is clear?
    noteOn(drumchan-1,this->note,velocity);
    this->offTime = millis() + holdtime;
  }
#else
  if(val >= threshold)
  {
    int velocity = 127; // full velocity
    this->state = NOTE_ON_STATE;
    noteOn(drumchan-1,this->note,velocity);
    this->offTime = millis() + holdtime;
  }
#endif
  //DEBUG("exit: checkInput()");
}

// turns the note off, if needed
void Trigger::checkNoteOff()
{
  //DEBUG("enter: checkNoteOff()");
  if(this->state != NOTE_ON_STATE) return;
  if(millis() >= this->offTime)
  {
    sendNoteOff();     
  }
  //DEBUG("exit: checkNoteOff()");
}

void Trigger::sendNoteOff()
{
  noteOff(drumchan-1, this->note);
  this->state = NOTE_OFF_STATE;
}


