#include "WProgram.h"
#include "Whammy.h"
#include "midi.h"
#include "qs7_scratchnoise.h"

extern int drumchan;

#define NOTE_OFF_STATE 0
#define WAIT_ON_VEL_STATE 1
#define NOTE_ON_STATE 2

#define THRESH 100

WhammyScratcher::WhammyScratcher(int analogPin)
{
  this->analogPin = analogPin;
  
  this->state = NOTE_OFF_STATE;
  this->offTime = 0;
  
  // get a useful rest value for monitoring purposes
//  int m = 0;
//  for(int i=0; i<100; i++)
//  {
//    int val = analogRead(analogPin);
//    if(val>m) m = val;
//  }
//  this->restValue = m;
    this->restValue = 450; // from experimenting
    
    this->lastValue = this->restValue;
}


// entry point for monitoring a Trigger
void WhammyScratcher::process()
{
  checkInput();
  checkNoteOff();
}

// checks the whammy to see if it is receiving a non-rest value
void WhammyScratcher::checkInput()
{
  //DEBUG("enter: checkInput()");
  if(this->state != NOTE_OFF_STATE) return; // don't check if we're currently playing (or waiting to calc the velocity)
  int val = analogRead(analogPin);
  

  if( ((val - lastValue) > THRESH) || ((lastValue - val) > THRESH) )
  {
    lastValue = val;
    int velocity = 100;
    this->state = NOTE_ON_STATE;
    int pitchShift = (val-restValue)*40;
    pitchBendMsg(drumchan-1, pitchShift);
    //Serial.print("whammy: ");
    //Serial.println(pitchShift);
    noteOn(drumchan-1, SCRATCH_NOISE_1, velocity);
    this->offTime = millis() + 100;
  }

}

// turns the note off, if needed
void WhammyScratcher::checkNoteOff()
{
  //DEBUG("enter: checkNoteOff()");
  if(this->state != NOTE_ON_STATE) return;
  if(millis() >= this->offTime)
  {
    sendNoteOff();     
  }
  //DEBUG("exit: checkNoteOff()");
}

void WhammyScratcher::sendNoteOff()
{
  noteOff(drumchan-1, SCRATCH_NOISE_1);
  resetPitchBend(drumchan-1);
  this->state = NOTE_OFF_STATE;
}
