#include "midi.h"
#include "electric_funkatron.h"
#include "WProgram.h"

//---------------------------------------------------------------------------
/*
 * Heavily modified from an original by:
 * Created 25 October 2006
 * copyleft 2006 Tod E. Kurt <tod@todbot.com
 * http://todbot.com/
 */

// Send a MIDI note-on message.  Like pressing a piano key
// channel ranges from 0-15
void noteOn(byte channel, byte note, byte velocity) {
  midiMsg( (0x90 | channel), note, velocity);
}

// Send a MIDI note-off message.  Like releasing a piano key
void noteOff(byte channel, byte note) {
  midiMsg( (0x80 | channel), note, 0);
}

// Send a general MIDI message
void midiMsg(byte cmd, byte data1, byte data2) 
{
  byte buf[3] = {cmd,data1,data2};
  Serial1.write(buf,3);
}

//--------------------------------------------------------------------------------
// By Lewis Baumstark
void pitchBendMsg(int channel, int change)
{
  unsigned int pitchMin = 0;
  unsigned int pitchMax = 16383;
  unsigned int pitchRest = 8192;
  
  unsigned int pitch = pitchRest + change;
  if(change > 0 && pitch < pitchRest) // overflow!
  {
    pitch = pitchMax;
  }
  else if(change < 0 && pitch > pitchRest) // underflow!
  {
    pitch = pitchMin;
  }
  else if(pitch > pitchMax)
  {
    pitch = pitchMax;
  }
  
  //Serial.print("pitch: ");
  //Serial.println(pitch);
  
  byte pitch_lsb = lowByte(pitch) & 0x7F; // only lower 7-bits considered
  byte pitch_msb = lowByte(pitch >> 7) & 0x7F; // msb is next-highest 7-bits
  midiMsg( (0xE0 | channel), pitch_lsb, pitch_msb);
}

void resetPitchBend(int channel)
{
  midiMsg((0xE0 | channel), 0x00, 0x40);
}
